import matplotlib.pyplot as plt
from scipy.io import wavfile
import os
from pydub import AudioSegment
import numpy as np
from tensorflow.keras.models import Model
from tensorflow.keras.layers import Dense, Activation, Dropout, Input, TimeDistributed, Conv1D
from tensorflow.keras.layers import GRU, BatchNormalization
from matplotlib.mlab import specgram

def Hmodel(input_shape):
    """
    Function creating the model's graph in Keras.
    
    Argument:
    input_shape -- shape of the model's input data (using Keras conventions)

    Returns:
    model -- Keras model instance
    with the help of Andrew ng and Hemant Kumar
    """
    
    X_input = Input(shape = input_shape)
    
 
    
    # Step 1: CONV layer (≈4 lines)
    X = Conv1D(196, kernel_size=15, strides=4)(X_input)                                 # CONV1D
    X = BatchNormalization()(X)                                 # Batch normalization
    X = Activation('relu')(X)                                 # ReLu activation
    X = Dropout(0.8)(X)                                 # dropout (use 0.8)

    # Step 2: First GRU Layer (≈4 lines)
    X = GRU(units = 128, return_sequences = True)(X) # GRU (use 128 units and return the sequences)
    X = Dropout(0.8)(X)                                 # dropout (use 0.8)
    X = BatchNormalization()(X)                                 # Batch normalization
    
    # Step 3: Second GRU Layer (≈4 lines)
    X = GRU(units = 128, return_sequences = True)(X)   # GRU (use 128 units and return the sequences)
    X = Dropout(0.8)(X)                                 # dropout (use 0.8)
    X = BatchNormalization()(X)                                  # Batch normalization
    X = Dropout(0.8)(X)                                  # dropout (use 0.8)
    
    # Step 4: Time-distributed dense layer (≈1 line)
    X = TimeDistributed(Dense(1, activation = "sigmoid"))(X) # time distributed  (sigmoid)


    model = Model(inputs = X_input, outputs = X)
    
    return model
# Calculate and plot spectrogram for a wav audio file
def graph_spectrogram(wav_file):
    rate, data = get_wav_info(wav_file)
    nfft = 200 # Length of each window segment
    fs = 8000 # Sampling frequencies
    noverlap = 120 # Overlap between windows
    nchannels = data.ndim
    if nchannels == 1:
        pxx, freqs, bins, im = plt.specgram(data, nfft, fs, noverlap = noverlap)
    elif nchannels == 2:
        pxx, freqs, bins, im = plt.specgram(data[:,0], nfft, fs, noverlap = noverlap)
    return pxx

# Load a wav file
def get_wav_info(wav_file):
    rate, data = wavfile.read(wav_file)
    return rate, data

# Used to standardize volume of audio clip
def match_target_amplitude(sound, target_dBFS):
    change_in_dBFS = target_dBFS - sound.dBFS
    return sound.apply_gain(change_in_dBFS)



def get_random_time_segment(segment_ms):
    """
    Gets a random time segment of duration segment_ms in a 10,000 ms audio clip.
    
    Arguments:
    segment_ms -- the duration of the audio clip in ms ("ms" stands for "milliseconds")
    
    Returns:
    segment_time -- a tuple of (segment_start, segment_end) in ms
    With Help of Andrew ng and logical spot
    """
    
    segment_start = np.random.randint(low=0, high=10000-segment_ms)   # Make sure segment doesn't run past the 10sec background 
    segment_end = segment_start + segment_ms - 1
    
    return (segment_start, segment_end)

def is_overlapping(segment_time, previous_segments):
    """
    Checks if the time of a segment overlaps with the times of existing segments.
    
    Arguments:
    segment_time -- a tuple of (segment_start, segment_end) for the new segment
    previous_segments -- a list of tuples of (segment_start, segment_end) for the existing segments
    
    Returns:
    True if the time segment overlaps with any of the existing segments, False otherwise
    """
    
    segment_start, segment_end = segment_time
    
    ### START CODE HERE ### (≈ 4 line)
    # Step 1: Initialize overlap as a "False" flag. (≈ 1 line)
    overlap = False
    
    # Step 2: loop over the previous_segments start and end times.
    # Compare start/end times and set the flag to True if there is an overlap (≈ 3 lines)
    for previous_start, previous_end in previous_segments:
        if segment_start <= previous_end and segment_end >= previous_start:
            overlap = True
    ### END CODE HERE ###

    return overlap

def insert_audio_clip(background, audio_clip, previous_segments):
    """
    Insert a new audio segment over the background noise at a random time step, ensuring that the 
    audio segment does not overlap with existing segments.
    
    Arguments:
    background -- a 10 second background audio recording.  
    audio_clip -- the audio clip to be inserted/overlaid. 
    previous_segments -- times where audio segments have already been placed
    
    Returns:
    new_background -- the updated background audio
    """
    
    # Get the duration of the audio clip in ms
    segment_ms = len(audio_clip)
    
    ### START CODE HERE ### 
    # Step 1: Use one of the helper functions to pick a random time segment onto which to insert 
    # the new audio clip. (≈ 1 line)
    segment_time = get_random_time_segment(segment_ms)
    
    # Step 2: Check if the new segment_time overlaps with one of the previous_segments. If so, keep 
    # picking new segment_time at random until it doesn't overlap. (≈ 2 lines)
    while is_overlapping(segment_time, previous_segments):
        segment_time = get_random_time_segment(segment_ms)

    # Step 3: Add the new segment_time to the list of previous_segments (≈ 1 line)
    previous_segments.append(segment_time)
    ### END CODE HERE ###
    
    # Step 4: Superpose audio segment and background
    new_background = background.overlay(audio_clip, position = segment_time[0])
    
    return new_background, segment_time



# GRADED FUNCTION: insert_ones

def insert_ones(y, segment_end_ms,tmy):
    """
    Update the label vector y. The labels of the 50 output steps strictly after the end of the segment 
    should be set to 1. By strictly we mean that the label of segment_end_y should be 0 while, the
    50 followinf labels should be ones.
    
    
    Arguments:
    y -- numpy array of shape (1, Ty), the labels of the training example
    segment_end_ms -- the end time of the segment in ms
    
    Returns:
    y -- updated labels
    """
    
    # duration of the background (in terms of spectrogram time-steps)
    segment_end_y = int(segment_end_ms * tmy/ 10000.0)
    
    # Add 1 to the correct index in the background label (y)
    ### START CODE HERE ### (≈ 3 lines)
    for i in range(segment_end_y + 1, segment_end_y + 51):
        if i < tmy:
            y[0, i] = 1
    ### END CODE HERE ###
    
    return y


def create_training_example(background, activates, negatives, tyc):
    """
    Creates a training example with a given background, activates, and negatives.
    
    Arguments:
    background -- a 10 second background audio recording
    activates -- a list of audio segments of the word "activate"
    negatives -- a list of audio segments of random words that are not "activate"
    
    Returns:
    x -- the spectrogram of the training example
    y -- the label at each time step of the spectrogram
    """
    
    # Set the random seed
    np.random.seed(18)
    
    # Make background quieter
    background = background - 20

    ### START CODE HERE ###
    # Step 1: Initialize y (label vector) of zeros (≈ 1 line)
    y = np.zeros((1, tyc))

    # Step 2: Initialize segment times as empty list (≈ 1 line)
    previous_segments = []
    ### END CODE HERE ###
    
    # Select 0-4 random "activate" audio clips from the entire list of "activates" recordings
    number_of_activates = np.random.randint(0, 5)
    random_indices = np.random.randint(len(activates), size=number_of_activates)
    random_activates = [activates[i] for i in random_indices]
    
    ### START CODE HERE ### (≈ 3 lines)
    # Step 3: Loop over randomly selected "activate" clips and insert in background
    for random_activate in random_activates:
        # Insert the audio clip on the background
        background, segment_time = insert_audio_clip(background, random_activate, previous_segments)
        # Retrieve segment_start and segment_end from segment_time
        segment_start, segment_end = segment_time
        # Insert labels in "y"
        y = insert_ones(y, segment_end_ms=segment_end,tmy=tyc)
    ### END CODE HERE ###

    # Select 0-2 random negatives audio recordings from the entire list of "negatives" recordings
    number_of_negatives = np.random.randint(0, 3)
    random_indices = np.random.randint(len(negatives), size=number_of_negatives)
    random_negatives = [negatives[i] for i in random_indices]

    ### START CODE HERE ### (≈ 2 lines)
    # Step 4: Loop over randomly selected negative clips and insert in background
    for random_negative in random_negatives:
        # Insert the audio clip on the background 
        background, _ = insert_audio_clip(background, random_negative, previous_segments)
    ### END CODE HERE ###
    
    # Standardize the volume of the audio clip 
    background = match_target_amplitude(background, -20.0)

    # Export new training example 
    file_handle = background.export("train" + ".wav", format="wav")
    print("File (train.wav) was saved in your directory.")
    
    # Get and plot spectrogram of the new recording (background with superposition of positive and negatives)
    x = graph_spectrogram("train.wav")
    
    return x, y




def chime_on_activate(filename, predictions, threshold, chime_file):
    audio_clip = AudioSegment.from_wav(filename)
    chime = AudioSegment.from_wav(chime_file)
    Ty = predictions.shape[1]
    # Step 1: Initialize the number of consecutive output steps to 0
    consecutive_timesteps = 0
    # Step 2: Loop over the output steps in the y
    for i in range(Ty):
        # Step 3: Increment consecutive output steps
        consecutive_timesteps += 1
        # Step 4: If prediction is higher than the threshold and more than 75 consecutive output steps have passed
        if predictions[0,i,0] > threshold and consecutive_timesteps > 75:
            # Step 5: Superpose audio and background using pydub
            audio_clip = audio_clip.overlay(chime, position = ((i / Ty) * audio_clip.duration_seconds)*1000)
            # Step 6: Reset consecutive output steps to 0
            consecutive_timesteps = 0
        
    audio_clip.export('chimeF', format='wav')


# Preprocess the audio to the correct format
def preprocess_audio(filename):
    # Trim or pad audio segment to 10000ms
    padding = AudioSegment.silent(duration=10000)
    segment = AudioSegment.from_wav(filename)[:10000]
    segment = padding.overlay(segment)
    # Set frame rate to 44100
    segment = segment.set_frame_rate(44100)
    # Export as wav
    segment.export(filename, format='wav')


def get_spectrogram(data):
    """
    Function to compute a spectrogram.
    
    Argument:
    predictions -- one channel / dual channel audio data as numpy array

    Returns:
    pxx -- spectrogram, 2-D array, columns are the periodograms of successive segments.
    """
    nfft = 200 # Length of each window segment
    fs = 8000 # Sampling frequencies
    noverlap = 120 # Overlap between windows
    nchannels = data.ndim
    if nchannels == 1:
        pxx, _, _ = specgram(data, nfft, fs, noverlap = noverlap)
    elif nchannels == 2:
        pxx, _, _ = specgram(data[:,0], nfft, fs, noverlap = noverlap)
    return pxx


def plt_spectrogram(data):
    """
    Function to compute and plot a spectrogram.
    
    Argument:
    predictions -- one channel / dual channel audio data as numpy array

    Returns:
    pxx -- spectrogram, 2-D array, columns are the periodograms of successive segments.
    """
    nfft = 200 # Length of each window segment
    fs = 8000 # Sampling frequencies
    noverlap = 120 # Overlap between windows
    nchannels = data.ndim
    if nchannels == 1:
        pxx, _, _, _ = plt.specgram(data, nfft, fs, noverlap = noverlap)
    elif nchannels == 2:
        pxx, _, _, _ = plt.specgram(data[:,0], nfft, fs, noverlap = noverlap)
    return pxx



from __future__ import print_function
import numpy as np
from pydub import AudioSegment
import random
import sys
import os
from scipy.io import wavfile
print("by Logical Spot")
import argparse
parser = argparse.ArgumentParser(description='Dir For Dataset e.g. neg an pos')
parser.add_argument('--inX', action='store', type=str, required=True)
parser.add_argument('--inY', action='store', type=str, required=True)
parser.add_argument('--epochs', action='store', type=int, required=True)
parser.add_argument('--tx', action='store', type=int, default=5511)
parser.add_argument('--bsize', action='store', type=int, default=10)
parser.add_argument('--nf', action='store', type=int, default=101)
parser.add_argument('--ty', action='store', type=int, default=1375)
args = parser.parse_args()


import matplotlib.pyplot as plt
from scipy.io import wavfile
import os
from pydub import AudioSegment

Tx = args.tx # The number of time steps input to the model from the spectrogram
n_freq = args.nf # Number of frequencies input to the model at each time step of the spectrogram
Ty = args.ty # The number of time steps in the output of our model

if os.path.isfile(args.inX):
    X = np.load(args.inX)
else:
    print('Provide correct input for inX!')

if os.path.isfile(args.inY):
    Y = np.load(args.inY)
else:
    print('Provide correct input for inX!')


print(X.shape)
print(Y.shape)
assert X.ndim == 3, "Error: X not have correct dimentions"
assert Y.ndim == 3, "Error: Y not have correct dimentions"
assert Y.shape[1] == Ty, "Error: Y not have correct dimentions"
assert X.shape[1] == Tx, "Error: X not have correct dimentions"


 
from sklearn.model_selection import train_test_split
X_train,X_test,Y_train,Y_test = train_test_split(X,Y,test_size=0.2,random_state=37)

 
#X_train.shape

 

from tensorflow.keras.models import Model
from tensorflow.keras.layers import Dense, Activation, Dropout, Input, TimeDistributed, Conv1D
from tensorflow.keras.layers import GRU, BatchNormalization
try:
    from tensorflow.keras.optimizers.legacy import Adam
except:
    from tensorflow.keras.optimizers import Adam



 
def model(input_shape):
    """
    Function creating the model's graph in Keras.
    
    Argument:
    input_shape -- shape of the model's input data (using Keras conventions)

    Returns:
    model -- Keras model instance
    with the help of Andrew ng and Hemant Kumar
    """
    
    X_input = Input(shape = input_shape)
    
 
    
    # Step 1: CONV layer (≈4 lines)
    X = Conv1D(196, kernel_size=15, strides=4)(X_input)                                 # CONV1D
    X = BatchNormalization()(X)                                 # Batch normalization
    X = Activation('relu')(X)                                 # ReLu activation
    X = Dropout(0.8)(X)                                 # dropout (use 0.8)

    # Step 2: First GRU Layer (≈4 lines)
    X = GRU(units = 128, return_sequences = True)(X) # GRU (use 128 units and return the sequences)
    X = Dropout(0.8)(X)                                 # dropout (use 0.8)
    X = BatchNormalization()(X)                                 # Batch normalization
    
    # Step 3: Second GRU Layer (≈4 lines)
    X = GRU(units = 128, return_sequences = True)(X)   # GRU (use 128 units and return the sequences)
    X = Dropout(0.8)(X)                                 # dropout (use 0.8)
    X = BatchNormalization()(X)                                  # Batch normalization
    X = Dropout(0.8)(X)                                  # dropout (use 0.8)
    
    # Step 4: Time-distributed dense layer (≈1 line)
    X = TimeDistributed(Dense(1, activation = "sigmoid"))(X) # time distributed  (sigmoid)


    model = Model(inputs = X_input, outputs = X)
    
    return model


 
model = model(input_shape = (Tx, n_freq))

 
model.summary()


 
opt = Adam(lr=0.0001, beta_1=0.9, beta_2=0.999, decay=0.01)
model.compile(loss='binary_crossentropy', optimizer=opt, metrics=["accuracy"])

epoch = args.epochs

model.fit(X,Y,batch_size=args.bsize,epochs=epoch)

# while True:
#     print("Wanna train more!! (y/n)")
#     ch = input('>> ')
#     if ch == 'y':
#         model.fit(X,Y,batch_size=5,epochs=5)
#     else:
#         break
 
model.save("model.h5")
print("Model Saved !")




import pathlib
Here = pathlib.Path(__file__).parent
import numpy as np
from pydub import AudioSegment
from pydub.playback import play
import sys
from . import hotword as ht
import argparse

from tensorflow.keras.models import Model, load_model

def detect_triggerword_spectrum(x):
    """
    Function to predict the location of the trigger word.
    
    Argument:
    x -- spectrum of shape (freqs, Tx)
    i.e. (Number of frequencies, The number time steps)

    Returns:
    predictions -- flattened numpy array to shape (number of output time steps)
    """
    # the spectogram outputs  and we want (Tx, freqs) to input into the model
    x  = x.swapaxes(0,1)
    x = np.expand_dims(x, axis=0)
    predictions = model.predict(x,verbose=0)
    return predictions.reshape(-1)

def has_new_triggerword(predictions, chunk_duration, feed_duration, threshold=0.5):
    """
    Function to detect new trigger word in the latest chunk of input audio.
    It is looking for the rising edge of the predictions data belongs to the
    last/latest chunk.
    
    Argument:
    predictions -- predicted labels from model
    chunk_duration -- time in second of a chunk
    feed_duration -- time in second of the input to model
    threshold -- threshold for probability above a certain to be considered positive

    Returns:
    True if new trigger word detected in the latest chunk
    """
    predictions = predictions > threshold
    chunk_predictions_samples = int(len(predictions) * chunk_duration / feed_duration)
    chunk_predictions = predictions[-chunk_predictions_samples:]
    level = chunk_predictions[0]
    for pred in chunk_predictions:
        if pred > level:
            return True
        else:
            level = pred
    return False


chunk_duration = 0.5 # Each read length in seconds from mic.
fs = 44100 # sampling rate for mic
chunk_samples = int(fs * chunk_duration) # Each read length in number of samples.
mic_index = 0
# Each model input data duration in seconds, need to be an integer numbers of chunk_duration
feed_duration = 10
feed_samples = int(fs * feed_duration)

assert feed_duration/chunk_duration == int(feed_duration/chunk_duration)

def get_audio_input_stream(callback):
    stream = pyaudio.PyAudio().open(
        format=pyaudio.paInt16,
        channels=1,
        rate=fs,
        input=True,
        frames_per_buffer=chunk_samples,
        input_device_index=mic_index,
        stream_callback=callback)
    return stream


import pyaudio
from queue import Queue
import sys
import time
silence_threshold = 150
# Queue to communiate between the audio callback and main thread
q = Queue()
# Data buffer for the input wavform
data = np.zeros(feed_samples, dtype='int16')

def callback(in_data, frame_count, time_info, status):
    global data, silence_threshold    
      
    data0 = np.frombuffer(in_data, dtype='int16')

    if np.abs(data0).mean() < silence_threshold:
        # print((np.abs(data0).mean()))
        return (in_data, pyaudio.paContinue)

    data = np.append(data,data0)    
    if len(data) > feed_samples:
        data = data[-feed_samples:]            # Process data async by sending a queue.
        q.put(data)
    return (in_data, pyaudio.paContinue)




def lsHotword_loop(mpath,silence_thres=100,chimePath=Here / "chime.wav",chimeOnWake=True,mic_idx=0,sampling_rate=44100):
    global silence_threshold, mic_index, model, chime, fs
    fs = sampling_rate
    mic_index = mic_idx
    chime = AudioSegment.from_wav(chimePath)
    silence_threshold = silence_thres
    model = load_model(mpath)
    stream = get_audio_input_stream(callback)
    stream.start_stream()
    
    try:
        while True:
            print("<<Waiting for Hotword>>")
            data = q.get()
            spectrum = ht.get_spectrogram(data)
            preds = detect_triggerword_spectrum(spectrum)
            new_trigger = has_new_triggerword(preds, chunk_duration, feed_duration)
            # sys.stdout.write('1')
            if new_trigger:
                sys.stdout.write('<<Activated>>')
                if chimeOnWake:
                    play(chime)
                stream.stop_stream()
                stream.close()
                return True
    except (KeyboardInterrupt, SystemExit):
       stream.stop_stream()
       stream.close()
       return False
        

class Hotword():
    def __init__(self,mpath,silence_thres=100,chimePath=Here / "chime.wav",chimeOnWake=True,mic_idx=0,sampling_rate=44100):
        global silence_threshold, mic_index, model, chime, fs
        fs = sampling_rate
        mic_index = mic_idx
        chime = AudioSegment.from_wav(chimePath)
        silence_threshold = silence_thres
        model = load_model(mpath)
    def HotwordLoop(self):
        stream = get_audio_input_stream(callback)
        stream.start_stream()
        
        try:
            while True:
                print("<<Waiting for Hotword>>")
                data = q.get()
                spectrum = ht.get_spectrogram(data)
                preds = detect_triggerword_spectrum(spectrum)
                new_trigger = has_new_triggerword(preds, chunk_duration, feed_duration)
                # sys.stdout.write('1')
                if new_trigger:
                    sys.stdout.write('<<Activated>>')
                    if chimeOnWake:
                        play(chime)
                    stream.stop_stream()
                    stream.close()
                    return True
        except (KeyboardInterrupt, SystemExit):
           stream.stop_stream()
           stream.close()
           return False



def HTest():
    parser = argparse.ArgumentParser(description='Model path e.g. "--model ./model.h5"')
    parser.add_argument('--model', action='store', type=str, required=True)
    args = parser.parse_args()
    print(lsHotword_loop(args.model))
    


from numpy.random import randint
from numpy import zeros,array
from numpy import save as npsave
from pydub import AudioSegment
import os
from scipy.io import wavfile
try:
    from tensorflow.keras.optimizers.legacy import Adam
except:
    from tensorflow.keras.optimizers import Adam
print("Full Hotword Data Generator and Trainer")
import argparse
from matplotlib.pyplot import specgram
from sklearn.model_selection import train_test_split
from . import hotword as ht


# Load a wav file
def get_wav_info(wav_file):
    rate, data = wavfile.read(wav_file)
    return rate, data

# Used to standardize volume of audio clip
def match_target_amplitude(sound, target_dBFS):
    change_in_dBFS = target_dBFS - sound.dBFS
    return sound.apply_gain(change_in_dBFS)

# Load raw audio files for speech synthesis
def load_raw_audio(pathD):
    activates = []
    backgrounds = []
    negatives = []
    for filename in os.listdir(pathD + "/positives/"):
        if filename.endswith("wav"):
            activate = AudioSegment.from_wav(pathD + "/positives/"+filename)
            activates.append(activate)
    for filename in os.listdir(pathD+"/backgrounds/"):
        if filename.endswith("wav"):
            background = AudioSegment.from_wav(pathD+"/backgrounds/"+filename)
            backgrounds.append(background)
    for filename in os.listdir(pathD + "/negatives/"):
        if filename.endswith("wav"):
            negative = AudioSegment.from_wav(pathD + "/negatives/"+filename)
            negatives.append(negative)
    return activates, negatives, backgrounds

def load_raw_audio_from_diff_path(posPath,negPath,bgPath):
    activates = []
    backgrounds = []
    negatives = []
    for filename in os.listdir(posPath):
        if filename.endswith("wav"):
            activate = AudioSegment.from_wav(os.path.join(posPath,filename))
            activates.append(activate)
    for filename in os.listdir(bgPath):
        if filename.endswith("wav"):
            background = AudioSegment.from_wav(os.path.join(bgPath,filename))
            backgrounds.append(background)
    for filename in os.listdir(negPath):
        if filename.endswith("wav"):
            negative = AudioSegment.from_wav(os.path.join(negPath,filename))
            negatives.append(negative)
    return activates, negatives, backgrounds


 
def get_random_time_segment(segment_ms):
    """
    Gets a random time segment of duration segment_ms in a 10,000 ms audio clip.
    
    Arguments:
    segment_ms -- the duration of the audio clip in ms ("ms" stands for "milliseconds")
    
    Returns:
    segment_time -- a tuple of (segment_start, segment_end) in ms
    """
    
    segment_start = randint(low=0, high=10000-segment_ms)   # Make sure segment doesn't run past the 10sec background 
    segment_end = segment_start + segment_ms - 1
    
    return (segment_start, segment_end)

 
def is_overlapping(segment_time, previous_segments):
    """
    Checks if the time of a segment overlaps with the times of existing segments.
    
    Arguments:
    segment_time -- a tuple of (segment_start, segment_end) for the new segment
    previous_segments -- a list of tuples of (segment_start, segment_end) for the existing segments
    
    Returns:
    True if the time segment overlaps with any of the existing segments, False otherwise
    """
    
    segment_start, segment_end = segment_time
    

    # Step 1: Initialize overlap as a "False" flag. (≈ 1 line)
    overlap = False
    
    # Step 2: loop over the previous_segments start and end times.
    # Compare start/end times and set the flag to True if there is an overlap (≈ 3 lines)
    for previous_start, previous_end in previous_segments:
        if segment_start <= previous_end and segment_end >= previous_start:
            overlap = True
   

    return overlap

 
def insert_audio_clip(background, audio_clip, previous_segments):
    """
    Insert a new audio segment over the background noise at a random time step, ensuring that the 
    audio segment does not overlap with existing segments.
    
    Arguments:
    background -- a 10 second background audio recording.  
    audio_clip -- the audio clip to be inserted/overlaid. 
    previous_segments -- times where audio segments have already been placed
    
    Returns:
    new_background -- the updated background audio
    """
    
    # Get the duration of the audio clip in ms
    segment_ms = len(audio_clip)
    
   
    # Step 1: Use one of the helper functions to pick a random time segment onto which to insert 
    # the new audio clip. (≈ 1 line)
    segment_time = get_random_time_segment(segment_ms)
    
    # Step 2: Check if the new segment_time overlaps with one of the previous_segments. If so, keep 
    # picking new segment_time at random until it doesn't overlap. (≈ 2 lines)
    while is_overlapping(segment_time, previous_segments):
        segment_time = get_random_time_segment(segment_ms)

    # Step 3: Add the new segment_time to the list of previous_segments (≈ 1 line)
    previous_segments.append(segment_time)

    
    # Step 4: Superpose audio segment and background
    new_background = background.overlay(audio_clip, position = segment_time[0])
    
    return new_background, segment_time

 
def insert_ones(y, segment_end_ms):
    """
    Update the label vector y. The labels of the 50 output steps strictly after the end of the segment 
    should be set to 1. By strictly we mean that the label of segment_end_y should be 0 while, the
    50 followinf labels should be ones.
    
    
    Arguments:
    y -- numpy array of shape (1, Ty), the labels of the training example
    segment_end_ms -- the end time of the segment in ms
    
    Returns:
    y -- updated labels
    """
    
    # duration of the background (in terms of spectrogram time-steps)
    segment_end_y = int(segment_end_ms * Ty / 10000.0)
    
    # Add 1 to the correct index in the background label (y)
   
    for i in range(segment_end_y + 1, segment_end_y + 51):
        if i < Ty:
            y[0, i] = 1
 
    
    return y

 
def create_training_example(background, activates, negatives):
    """
    Creates a training example with a given background, activates, and negatives.
    
    Arguments:
    background -- a 10 second background audio recording
    activates -- a list of audio segments of the word "activate"
    negatives -- a list of audio segments of random words that are not "activate"
    
    Returns:
    x -- the spectrogram of the training example
    y -- the label at each time step of the spectrogram
    """
    
    # Set the random seed
   
    
    # Make background quieter
    background = background - 20

  
    # Step 1: Initialize y (label vector) of zeros (≈ 1 line)
    y = zeros((1, Ty))

    # Step 2: Initialize segment times as empty list (≈ 1 line)
    previous_segments = []
 
    
    # Select 0-4 random "activate" audio clips from the entire list of "activates" recordings
    number_of_activates = randint(0, 5)
    random_indices = randint(len(activates), size=number_of_activates)
    random_activates = [activates[i] for i in random_indices]
    
  
    # Step 3: Loop over randomly selected "activate" clips and insert in background
    for random_activate in random_activates:
        # Insert the audio clip on the background
        background, segment_time = insert_audio_clip(background, random_activate, previous_segments)
        # Retrieve segment_start and segment_end from segment_time
        segment_start, segment_end = segment_time
        # Insert labels in "y"
        y = insert_ones(y, segment_end_ms=segment_end)
    

    # Select 0-2 random negatives audio recordings from the entire list of "negatives" recordings
    number_of_negatives = randint(0, 3)
    random_indices = randint(len(negatives), size=number_of_negatives)
    random_negatives = [negatives[i] for i in random_indices]

   
    # Step 4: Loop over randomly selected negative clips and insert in background
    for random_negative in random_negatives:
        # Insert the audio clip on the background 
        background, _ = insert_audio_clip(background, random_negative, previous_segments)
   
    
    # Standardize the volume of the audio clip 
    background = match_target_amplitude(background, -20.0)

    # Export new training example 
    file_handle = background.export("train" + ".wav", format="wav")
    print("File (train.wav) was saved in your directory.")
    
    # Get and plot spectrogram of the new recording (background with superposition of positive and negatives)
    x = ht.graph_spectrogram("train.wav")
    print(y)
    return x, y

 

def main():
    global Ty
    parser = argparse.ArgumentParser(description='Dir For Dataset e.g. neg an pos')
    parser.add_argument('--input', action='store', type=str, required=True)
    parser.add_argument('--epochs', action='store', type=int, required=True)
    parser.add_argument('--tx', action='store', type=int, default=5511) # default for 44100Hz 10sec audio file
    parser.add_argument('--nsamp', action='store', type=int, default=32) 
    parser.add_argument('--nf', action='store', type=int, default=101)  # default for 44100Hz audio file
    parser.add_argument('--ty', action='store', type=int, default=1375) # default for 44100Hz audio file
    parser.add_argument('--bsize', action='store', type=int, default=10) # default for 44100Hz audio file
    args = parser.parse_args()
    Ty = args.ty # The number of time steps in the output of our model
    TrainHotwordModel(pathD=args.input,epochs=args.epochs,Tx=args.tx,n_freq=args.nf,nsamples=args.nsamp,ty=args.ty,batch_size=args.bsize)
    

def TrainHotwordModel(pathD,epochs,Tx=5511,n_freq=101,nsamples=30,ty=1375,batch_size=10):
    global Ty
    Ty = ty
    activates, negatives, backgrounds = load_raw_audio(pathD=pathD)
    X = []
    Y = []
    for i in range(0, nsamples):
        if i%10 == 0:
            print(i)
        x, y = create_training_example(backgrounds[i % 2], activates, negatives)
        X.append(x.swapaxes(0,1))
        Y.append(y.swapaxes(0,1))
    X=array([X])
    X=X[0]
    Y=array([Y])
    Y=Y[0]
    print(X.shape)
    print(X.ndim)
    print(Y.shape)
    print(Y.ndim)
    npsave('./X.npy',X)
    npsave('./Y.npy',Y)
    print("Successfull!!")
    assert X.ndim == 3, "Error: X not have correct dimentions"
    assert Y.ndim == 3, "Error: Y not have correct dimentions"
    assert Y.shape[1] == Ty, "Error: Y not have correct dimentions"
    assert X.shape[1] == Tx, "Error: X not have correct dimentions"
    X_train,X_test,Y_train,Y_test = train_test_split(X,Y,test_size=0.2,random_state=37)
    model = ht.Hmodel(input_shape = (Tx, n_freq))
    model.summary()
    opt = Adam(lr=0.0001, beta_1=0.9, beta_2=0.999, decay=0.01)
    model.compile(loss='binary_crossentropy', optimizer=opt, metrics=["accuracy"])
    model.fit(X,Y,batch_size=batch_size,epochs=epochs)
    model.save("model.h5")
    print("Model Saved !")



from numpy import load as npload
from os import path
print("lsHotwordTrainer")
import argparse
from sklearn.model_selection import train_test_split
try:
    from tensorflow.keras.optimizers.legacy import Adam
except:
    from tensorflow.keras.optimizers import Adam
from . import hotword as ht




def main():
    parser = argparse.ArgumentParser(description='Dir For Dataset e.g. neg an pos')
    parser.add_argument('--inX', action='store', type=str, required=True)
    parser.add_argument('--inY', action='store', type=str, required=True)
    parser.add_argument('--epochs', action='store', type=int, required=True)
    parser.add_argument('--tx', action='store', type=int, default=5511)
    parser.add_argument('--nf', action='store', type=int, default=101)
    parser.add_argument('--ty', action='store', type=int, default=1375)
    parser.add_argument('--bsize', action='store', type=int, default=10)
    args = parser.parse_args()
    Tx = args.tx # The number of time steps input to the model from the spectrogram
    n_freq = args.nf # Number of frequencies input to the model at each time step of the spectrogram
    Ty = args.ty # The number of time steps in the output of our model
    if path.isfile(args.inX):
        X = npload(args.inX)
    else:
        print('Provide correct input for inX!')
    
    if path.isfile(args.inY):
        Y = npload(args.inY)
    else:
        print('Provide correct input for inX!')
    print(X.shape)
    print(Y.shape)
    assert X.ndim == 3, "Error: X not have correct dimentions"
    assert Y.ndim == 3, "Error: Y not have correct dimentions"
    assert Y.shape[1] == Ty, "Error: Y not have correct dimentions"
    assert X.shape[1] == Tx, "Error: X not have correct dimentions"
    X_train,X_test,Y_train,Y_test = train_test_split(X,Y,test_size=0.2,random_state=37)
    model = ht.Hmodel(input_shape = (Tx, n_freq))
    model.summary()
    opt = Adam(lr=0.0001, beta_1=0.9, beta_2=0.999, decay=0.01)
    model.compile(loss='binary_crossentropy', optimizer=opt, metrics=["accuracy"])
    epoch = args.epochs
    model.fit(X,Y,batch_size=args.bsize,epochs=epoch)
    model.save("model.h5")
    print("Model Saved !")

 




from setuptools import setup, find_packages
from os import path

HERE = path.abspath(path.dirname(__file__))
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
  name = 'lsHotword',
  packages = ['lsHotword'],
  version = '1.2.3',
  license='MIT',
  include_package_data=True,
  long_description=long_description,
  long_description_content_type='text/markdown',
  description = 'Hotword/Wake Word detection in python for all platforms(Windows/Linux/Mac).',
  author = 'Hemant Kumar',
  author_email = 'iamhemantindia@protonmail.com',
  url = 'https://github.com/HemantKArya/lsHotword',
  download_url = '',
  package_data={
        'lsHotword': ['*.wav'],
    },
  keywords = ['hotword', 'detector', 'lshotword','windows','python','wake word','wake-word','detection'],
  install_requires=[
          'matplotlib',
          'numpy',
          'pydub',
          'pyaudio',
          'tensorflow',
          'scipy',
          'pyqt6'
      ],
  entry_points={
        'console_scripts': ['lshTrainer = lsHotword.funcHTrainer:main',
                            'lshDatagen = lsHotword.funcHDatagen:main',
                            'lshTestModel = lsHotword.ls:HTest',
                            'lshUITrainer = lsHotword.uitrainer:main',
                            'lshModelTrainer = lsHotword.funcHfTrainModel:main']
    },
  classifiers=[],
)